extends Node2D

export var horizontal_elements = 5
export var vertical_elements = 4

export var starting_elements = 2

const texture_size_x = 512
const texture_size_y = 512

var horizontal_step
var vertical_step
var horizontal_offset
var vertical_offset

const template = "element_%s-%s"

const NewElement = preload('res://scenes/element.tscn')
var all_elements = {}

var score = 0
var score_display
export var house_heart_bonus = 100
export var end_element_penalty = -50
export var click_penalty = -5
export var chain_bonus = 1
export var time_bonus = 1
export var time_bonus_chain = 2

var chain = 0

export var total_time = 45
var game_time 
var time_display

var one_heart = false
var one_home = false
var one_home_heart = false
var minimum_elements = true;

var game_over_display
var game_over = false
var restart_button

var elements_left = 0

# Called when the node enters the scene tree for the first time.
func _ready():
	
	print("\n")
	
	adjust_board()
	
	create_elements()
	
	set_starting_elements()
	
	game_time = get_parent().get_node("Timer")
	game_time.set_wait_time(float(total_time))
	game_time.set_one_shot(true)
	game_time.start()
	
	score_display = get_parent().get_node("ScoreDisplay")
	time_display = get_parent().get_node("TimeDisplay")
	game_over_display = get_parent().get_node("GameOverDisplay")
	
	restart_button = get_parent().get_node("RestartButton")
	restart_button.visible = false
	
	update_score(0)
	
	print_board_console()

# Called every frame. 'delta' is the elapsed time since the previous frame.
func _process(delta):
	if !game_over:
		check_clicked()
		display_time()
		check_minimum_elements()
		check_gameover()

func adjust_board():
	horizontal_step = get_viewport().size.x/horizontal_elements
	vertical_step = get_viewport().size.y/vertical_elements * 0.75
	horizontal_offset = -texture_size_x/horizontal_elements
	vertical_offset = texture_size_y/vertical_elements * 0.65

func create_elements():
	# instantiate the elements and set their position
	for i in range(1, vertical_elements + 1):
		for j in range(1, horizontal_elements + 1): 
			
			var new_element = (template % [j, i])
			all_elements[new_element] = NewElement.instance()
			add_child(all_elements[new_element])
			all_elements[new_element].set_name(new_element)
			
			all_elements[new_element].position.x = horizontal_offset + j * horizontal_step
			all_elements[new_element].position.y = vertical_offset + i * vertical_step
			
			all_elements[new_element].change_state(0)

func set_starting_elements():
	var first_x = 0
	var first_y = 0
	var random_x = 0
	var random_y = 0
	
	#get 2 random coordinates for X starting elements and set their state
	for i in range (1, starting_elements + 1):
		randomize()
		random_x = randi() % horizontal_elements + 1
		random_y = randi() % vertical_elements + 1
		
		if i == 1:
			first_x = random_x
			first_y = random_y
		
		elif i == 2:
			while ((random_x == first_x) and ((random_y == first_y + 1) or(random_y == first_y - 1))) or \
			      ((random_y == first_y) and ((random_x == first_x + 1) or(random_x == first_x - 1))) or \
				  ((random_x == first_x) and (random_y == first_y)):
				
				if ((random_x == first_x) and (random_y == first_y)):
					print("2nd Element on same position than 1st, randomizing again!")
				if ((random_x == first_x) and ((random_y == first_y + 1) or(random_y == first_y - 1))) or \
			       ((random_y == first_y) and ((random_x == first_x + 1) or(random_x == first_x - 1))):
					print("2nd Element is neighbour from 1st, randomizing again!")
				
				random_x = randi() % horizontal_elements + 1
				random_y = randi() % vertical_elements + 1
		
		var temp_element = all_elements[template % [random_x, random_y]]
		
		if i == 1:
			temp_element.change_state(1)
		elif i == 2:
			temp_element.change_state(2)
		else:
			temp_element.change_state(randi() % 2 + 1)
		
		print ("Starting Element " + str(i) + ": " + temp_element.name + " - State: " + str(temp_element.state))

func check_clicked():
	for i in range(1, vertical_elements + 1):
		for j in range(1, horizontal_elements + 1):
			if (all_elements[template % [j, i]].clicked) == true:
				print ("Clicked Element: " + all_elements[template % [j, i]].name + \
				       " - State: " + str(all_elements[template % [j, i]].state))
				all_elements[template % [j, i]].set_clicked(false)
				update_score(click_penalty)
				whack(j,i)
				print_board_console()

func whack(var x, y):
	if all_elements[template % [x, y]].state != 0:
		
		if all_elements[template % [x, y]].state != 3:
			change_neighbours(x, y, all_elements[template % [x, y]].state)
			chain = 0
		else:
			update_score(house_heart_bonus + house_heart_bonus * chain * chain_bonus)
			print("\nChain: " + str(chain))
			print("Chain Bonus: " + str(house_heart_bonus * chain * chain_bonus))
			if chain >= time_bonus_chain:
				var time_left = game_time.time_left
				game_time.stop()
				game_time.set_wait_time(float(time_left + time_bonus))
				game_time.start()
				print("\nBonus Time: " + str(time_bonus))
			chain += 1
		
		get_parent().get_node("ChalkSound").play()
		all_elements[template % [x, y]].change_state(0)

func change_neighbours(var x, y, new_state):
	if x > 1:
		all_elements[template % [x-1, y]].change_state(new_state)
	if x < horizontal_elements:
		all_elements[template % [x+1, y]].change_state(new_state)
	if y > 1:
		all_elements[template % [x, y-1]].change_state(new_state)
	if y < vertical_elements:
		all_elements[template % [x, y+1]].change_state(new_state)

func update_score(var value):
	score += value
	if score < 0:
		score =0
	score_display.text = str(score) #"Score: " + str(score)
	print("\nScore: " + str(score))

func display_time():
	time_display.text = str(round(game_time.time_left)) # "Time: " + str(round(game_time.time_left))

func check_minimum_elements():
	one_heart = false
	one_home = false
	one_home_heart = false
	
	for i in range(1, vertical_elements + 1):
		for j in range(1, horizontal_elements + 1):
			if (all_elements[template % [j, i]].state) == 1:
				one_heart = true
			if (all_elements[template % [j, i]].state) == 2:
				one_home = true
			if (all_elements[template % [j, i]].state) == 3:
				one_home_heart = true
	
	if !one_heart or !one_home:
		if !one_home_heart:
				minimum_elements = false

func count_elements():
	for i in range(1, vertical_elements + 1):
		for j in range(1, horizontal_elements + 1):
			if (all_elements[template % [j, i]].state) == 1:
				elements_left += 1;
			if (all_elements[template % [j, i]].state) == 2:
				elements_left += 1;
	print("Elements Left: " + str(elements_left))

func check_gameover():
	if game_time.time_left <= 0 or !minimum_elements:
		game_over_display.text = "GAME OVER"
		count_elements()
		print("End Game Penalty: " + str(elements_left * end_element_penalty))
		update_score(elements_left * end_element_penalty)
		restart_button.visible = true
		game_over = true

func print_board_console():
	print("\nPrinting Board:")
	for i in range(1, vertical_elements + 1):
		var element_line = ""
		for j in range(1, horizontal_elements + 1):
			element_line += str(all_elements[template % [j, i]].state)
		print(element_line)
	print("\n")
	
func _on_RestartButton_pressed():
	get_tree().reload_current_scene()